%skeleton "lalr1.cc"
%require "3.2"
%language "c++"
%locations
%defines

%define api.namespace { gen }
%define api.parser.class { Parser }
%define api.token.constructor
%define api.value.type variant

%define parse.error verbose
%define parse.lac full


// **** You must have parse.trace defined otherwise we get a
// **** Heap Corrupted error using MSVC

//%define parse.trace

// ****


%code requires
{
    #pragma warning(push)
    #pragma warning(disable : 4065) // switch statement contains 'default' but no 'case' labels
}

%code top
{
    #define YYDEBUG 1
    #include "driver.h"
    #include "scanner.h"
    #define scanner (*pDriver->getScanner())
}

%code
{ 
    gen::Parser::symbol_type yylex(Driver * pDriver) {
        return pDriver->getScanner()->get_next_token();
    }
}

%parse-param { Driver * pDriver }
%lex-param { Driver * pDriver }

%token <gen::generic_t> IDENTIFIER CONSTANT TYPENAME

%token AUTO CASE_ DEFAULT IF THEN ELSE SWITCH WHILE DO FOR GOTO CONTINUE BREAK RETURN
%token TYPEDEF EXTERN STATIC AUTO_ REGISTER SIZEOF INLINE
%token INT8_T INT16_T INT32_T INT64_T UINT8_T UINT16_T UINT32_T UINT64_T
%token STRUCT UNION ENUM ELLIPSIS

%token CHAR SHORT INT_ LONG_ SIGNED UNSIGNED FLOAT DOUBLE CONST_ VOLATILE VOID_
%token PTR_TO INC DEC SHL SHR LE GE EQ NE MUL_EQ DIV_EQ MOD_EQ ADD_EQ
%token SUB_EQ SHL_EQ SHR_EQ AND_EQ XOR_EQ OR_EQ ANDAND OROR

%start start-rule

%%

start-rule:
	declaration ';'
    | start-rule declaration ';'
	;

// C-style storage declaration
declaration:
    storage_type decl
    ;

storage_type:
    VOID_
    | CHAR
    | SHORT
    | LONG_
    | SIGNED
    | UNSIGNED
    | INT_
    | FLOAT
    | DOUBLE
    | UINT8_T | UINT16_T | UINT32_T | UINT64_T 
    | INT8_T | INT16_T | INT32_T | INT64_T 
    | IDENTIFIER;   // This is a typedef'd type
//    | IDENTIFIER '(' ')'
//    | IDENTIFIER '(' expr ')'
    ;

decl:
    pointer_to decl_name
    | decl_name
    ;

pointer_to:
	'*'
	| '*' type_qualifier_list
	| '*' pointer_to
	| '*' type_qualifier_list pointer_to
	;
    
decl_name:
    '(' decl ')'
    | '(' decl ')' '[' ']'
    | '(' decl ')' '[' expr ']'
    | IDENTIFIER
    | IDENTIFIER '[' ']'
    | IDENTIFIER '[' expr ']'
    ;

type_qualifier_list:
    type_qualifier
    | type_qualifier_list type_qualifier
    ;

type_qualifier:
    CONST_
    | VOLATILE
    ;

expr:
    CONSTANT
    ;

%%

/* Epilogue */

#pragma warning(pop)

// Bison expects us to provide implementation - 
// otherwise linker complains about an unresolved external symbol
void gen::Parser::error(const location &loc , const std::string &message)
{
    scanner.yyout << '[' << loc.begin.line << ':' << loc.begin.column;

    if (loc.begin.line == loc.end.line)
        scanner.yyout << '-' << loc.end.column << ']';
    else {
        scanner.yyout << "][" << loc.end.line << ':' << loc.end.column << ']';
    }
    scanner.yyout << " Error: " << message << std::endl;
}
