/*
 * Scanner for the GEN language
 */

%top{
	#pragma warning(push)
	#pragma warning(disable : 4005) // '(U)INT*_(MIN/MAX)': macro redefinition
}

%{
    #define YYDEBUG 1
    #include "driver.h"
	#include "scanner.h"

	#define yyterminate() return gen::Parser::make_YYEOF(gen::location());
	# define YY_USER_ACTION  loc.columns (yyleng);
%}

%option nodefault
%option c++
%option yyclass="Scanner"
%option prefix="gen"
%option noyywrap

%option yylineno
 //%option case-insensitive
%option nounistd
%option stack

%x COMMENT

wspace				[ \t]+
inum				[0-9]+
alpha				[A-Za-z_]+
ident				{alpha}({alpha}|{inum})*
hnum				[0-9A-F]+
exp					[eE][+-]?{inum}+
str					\'([^'\n]|\'\')+\'
bad_str		        \'([^'\n]|\'\')+


%x SC_COMMENT

 // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%

%{
  // A handy shortcut to the location held by the driver.
//  location& loc = pDriver->pScanner->loc;
  // Code run each time yylex is called.
  loc.step ();
  int comment_nesting = 0;  /* Line 4 */
%}

"/*"             { BEGIN(SC_COMMENT); }
<SC_COMMENT>{
  "/*"           { ++comment_nesting; }
  "*"+"/"        { if (comment_nesting) --comment_nesting;
                   else BEGIN(INITIAL); }
  "*"+           ; /* Line 11 */
  [^/*\n]+       ; /* Line 12 */
  [/]            ; /* Line 13 */
  \n             ; /* Line 14 */
}

auto { return gen::Parser::make_AUTO_(loc); }
break { return gen::Parser::make_BREAK(loc); }
case { return gen::Parser::make_CASE_(loc); }
char { return gen::Parser::make_CHAR(loc); }
const { return gen::Parser::make_CONST_(loc); }
continue { return gen::Parser::make_CONTINUE(loc); }
default { return gen::Parser::make_DEFAULT(loc); }
do { return gen::Parser::make_DO(loc); }
double { return gen::Parser::make_DOUBLE(loc); }
else { return gen::Parser::make_ELSE(loc); }
enum { return gen::Parser::make_ENUM(loc); }
extern { return gen::Parser::make_EXTERN(loc); }
float { return gen::Parser::make_FLOAT(loc); }
for { return gen::Parser::make_FOR(loc); }
goto { return gen::Parser::make_GOTO(loc); }
if { return gen::Parser::make_IF(loc); }
inline { return gen::Parser::make_INLINE(loc); }
int { return gen::Parser::make_INT_(loc); }
int8_t { return gen::Parser::make_INT_(loc); }
int16_t { return gen::Parser::make_INT_(loc); }
int32_t { return gen::Parser::make_INT_(loc); }
int64_t { return gen::Parser::make_INT_(loc); }
long { return gen::Parser::make_LONG_(loc); }
register { return gen::Parser::make_REGISTER(loc); }
return { return gen::Parser::make_RETURN(loc); }
short { return gen::Parser::make_SHORT(loc); }
signed { return gen::Parser::make_SIGNED(loc); }
sizeof { return gen::Parser::make_SIZEOF(loc); }
static { return gen::Parser::make_STATIC(loc); }
struct { return gen::Parser::make_STRUCT(loc); }
switch { return gen::Parser::make_SWITCH(loc); }
then { return gen::Parser::make_THEN(loc); }
typedef { return gen::Parser::make_TYPEDEF(loc); }
uint8_t { return gen::Parser::make_INT8_T(loc); }
uint16_t { return gen::Parser::make_INT16_T(loc); }
uint32_t { return gen::Parser::make_INT32_T(loc); }
uint64_t { return gen::Parser::make_INT64_T(loc); }
union { return gen::Parser::make_UNION(loc); }
unsigned { return gen::Parser::make_UNSIGNED(loc); }
void { return gen::Parser::make_VOID_(loc); }
volatile { return gen::Parser::make_VOLATILE(loc); }
while { return gen::Parser::make_WHILE(loc); }


   /* numbers */

{inum}	{ int64_t number = strtol(yytext, NULL, 10); return gen::Parser::make_CONSTANT(gen::generic_t(number), loc); }
{hnum}	{ int number = strtol(yytext, NULL, 16); return gen::Parser::make_CONSTANT(gen::generic_t(number), loc); }
0?[bB][01]+' { int number = strtol(yytext, NULL, 2); return gen::Parser::make_CONSTANT(gen::generic_t(number), loc); }


{inum}\./[^\.eE0-9] | {inum}\.{inum}{exp}? | {inum}\.{exp} | \.{inum}{exp}? { double number = std::stod(yytext); return gen::Parser::make_CONSTANT(gen::generic_t(number), loc); }
'(\\.|''|[^'\n])*'   | \"(\\.|\"\"|[^"\n])*\"  { return gen::Parser::make_CONSTANT(gen::generic_t(yytext), loc); }

{ident}  { return gen::Parser::make_IDENTIFIER(gen::generic_t(yytext), loc); }

"||"	{ return gen::Parser::make_OROR(loc); }
"&&"	{ return gen::Parser::make_ANDAND(loc); }
"<="	{ return gen::Parser::make_LE(loc); }
"=="	{ return gen::Parser::make_EQ(loc); }
"!="	{ return gen::Parser::make_NE(loc); }
">="	{ return gen::Parser::make_GE(loc); }

\/\/.*    continue;

<*>[ \t\r]+ { loc.columns(yyleng); loc.step(); }
<*>[\n]+	{ loc.lines(yyleng); loc.step(); }
<*>.		{ return gen::Parser::symbol_type(*yytext, loc); }

%%

#pragma warning(pop)

gen::Scanner::~Scanner()
{
	yy_delete_buffer(YY_CURRENT_BUFFER);
}
